"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getCacheDB = exports.CLIError = undefined;

let getCacheDB = exports.getCacheDB = (() => {
  var _ref = _asyncToGenerator(function* (projectPath, loadState) {
    const configPath = _path2.default.join(_os2.default.homedir(), '.sb-babel-cli', `cache-timestamps-${getSha1(projectPath)}`);

    const adapter = new _FileAsync2.default(configPath, {
      serialize: JSON.stringify
    });
    adapter.write = (0, _debounce2.default)(adapter.write, 1000);
    const db = yield (0, _lowdb2.default)(adapter);

    if (!loadState) {
      yield db.setState({});
    }

    return db;
  });

  return function getCacheDB(_x, _x2) {
    return _ref.apply(this, arguments);
  };
})();

exports.getSha1 = getSha1;
exports.logError = logError;
exports.getBabelTransformFile = getBabelTransformFile;

var _os = require("os");

var _os2 = _interopRequireDefault(_os);

var _lowdb = require("lowdb");

var _lowdb2 = _interopRequireDefault(_lowdb);

var _pify = require("pify");

var _pify2 = _interopRequireDefault(_pify);

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _crypto = require("crypto");

var _crypto2 = _interopRequireDefault(_crypto);

var _debounce = require("lodash/debounce");

var _debounce2 = _interopRequireDefault(_debounce);

var _resolveFrom = require("resolve-from");

var _resolveFrom2 = _interopRequireDefault(_resolveFrom);

var _FileAsync = require("lowdb/adapters/FileAsync");

var _FileAsync2 = _interopRequireDefault(_FileAsync);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

class CLIError extends Error {}

exports.CLIError = CLIError;

function getSha1(contents) {
  const hash = _crypto2.default.createHash('sha1');

  hash.update(contents);
  return hash.digest('hex');
}

function logError(error) {
  if (error instanceof CLIError) {
    console.error('ERROR', error.message);
  } else {
    console.error(error);
  }
}

let transformFileCached;

function getBabelTransformFile(projectPath) {
  if (!transformFileCached) {
    let resolved;

    try {
      resolved = (0, _resolveFrom2.default)(projectPath, '@babel/core');
    } catch (_) {
      /* No Op */
    }

    if (!resolved) {
      throw new CLIError('Unable to find @babel/core in your project');
    } // eslint-disable-next-line global-require,import/no-dynamic-require


    const babelCore = require(resolved);

    transformFileCached = (0, _pify2.default)(babelCore.transformFile);
  }

  return transformFileCached;
}