"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

let main = (() => {
  var _ref = _asyncToGenerator(function* (config) {
    let processFile = (() => {
      var _ref2 = _asyncToGenerator(function* (sourceFile, outputFile, stats) {
        if (!sourceFile.endsWith('.js')) return;
        const transformed = yield babelTransformFile(sourceFile, {
          root: config.root
        });
        yield (0, _makeDir2.default)(_path2.default.dirname(outputFile));
        yield _sbFs2.default.writeFile(outputFile, transformed.code, {
          mode: stats.mode
        });
        log(sourceFile, '->', outputFile);

        if (config.writeFlowSources) {
          const flowOutputFile = `${outputFile}.flow`;

          try {
            yield _sbFs2.default.unlink(flowOutputFile);
          } catch (_) {
            /* No Op */
          }

          try {
            yield _sbFs2.default.symlink(_path2.default.resolve(sourceFile), flowOutputFile);
            log(sourceFile, '->', flowOutputFile);
          } catch (error) {
            /* No Op */
          }
        }

        timestampCache.set((0, _helpers.getSha1)(sourceFile), stats.mtime.getTime()).write();
      });

      return function processFile(_x2, _x3, _x4) {
        return _ref2.apply(this, arguments);
      };
    })();

    let spawnedProcess;
    const timestampCache = yield (0, _helpers.getCacheDB)(config.sourceDirectory, !config.disableCache);
    const babelTransformFile = (0, _helpers.getBabelTransformFile)(config.sourceDirectory);
    const transformationQueue = new _pQueue2.default({
      concurrency: _os2.default.cpus().length
    });

    function log(...items) {
      if (config.execute) {
        console.log(`${_chalk2.default.yellow('[sb-babel-cli]')}`, ...items);
      } else {
        console.log(...items);
      }
    }

    function execute() {
      if (!config.execute) return;

      if (!spawnedProcess) {
        log(_chalk2.default.yellow(_package2.default.version));
        log(_chalk2.default.yellow('to restart at any time, enter `rs`'));
      }

      log(_chalk2.default.green(`starting 'node ${config.execute}'`));

      if (spawnedProcess) {
        spawnedProcess.kill();
      }

      spawnedProcess = _child_process2.default.spawn(process.execPath, [config.execute], {
        stdio: 'inherit'
      });
    }

    const debounceExecute = (0, _debounce2.default)(function () {
      try {
        execute();
      } catch (error) {
        (0, _helpers.logError)(error);
      }
    }, config.executeDelay);
    yield (0, _iterate2.default)({
      rootDirectory: config.sourceDirectory,
      sourceDirectory: config.sourceDirectory,
      outputDirectory: config.outputDirectory,
      ignored: config.ignored,
      keepExtraFiles: config.keepExtraFiles,
      filesToKeep: function filesToKeep(input) {
        return input.concat(config.writeFlowSources ? input.map(function (i) {
          return `${i}.flow`;
        }) : []);
      },

      callback(sourceFile, outputFile, stats) {
        return _asyncToGenerator(function* () {
          const cachedTimestamp = yield timestampCache.get((0, _helpers.getSha1)(sourceFile)).value();

          if (cachedTimestamp === stats.mtime.getTime()) {
            if (!config.execute) {
              log(sourceFile, 'is unchanged');
            }

            return;
          }

          transformationQueue.add(function () {
            return processFile(sourceFile, outputFile, stats);
          }).catch(_helpers.logError);
        })();
      }

    });
    if (!config.watch) return;

    if (config.execute && process.stdin) {
      if (typeof process.stdin.unref === 'function') {
        process.stdin.unref();
      }

      process.stdin.on('data', function (chunk) {
        if (chunk.toString().trim() === 'rs') {
          debounceExecute();
        }
      });
    }

    const resolvedSourceDirectory = _path2.default.resolve(config.sourceDirectory);

    const watcher = _chokidar2.default.watch(resolvedSourceDirectory, {
      ignored: config.ignored,
      alwaysStat: true,
      ignoreInitial: true
    });

    watcher.on('add', function (givenFileName, stats) {
      const fileName = _path2.default.relative(resolvedSourceDirectory, givenFileName);

      const sourceFile = _path2.default.join(config.sourceDirectory, fileName);

      const outputFile = _path2.default.join(config.outputDirectory, fileName);

      transformationQueue.add(() => processFile(sourceFile, outputFile, stats)).catch(_helpers.logError).then(debounceExecute);
    });
    watcher.on('change', function (givenFileName, stats) {
      const fileName = _path2.default.relative(resolvedSourceDirectory, givenFileName);

      const sourceFile = _path2.default.join(config.sourceDirectory, fileName);

      const outputFile = _path2.default.join(config.outputDirectory, fileName);

      transformationQueue.add(() => processFile(sourceFile, outputFile, stats)).catch(_helpers.logError).then(debounceExecute);
    });
    watcher.on('unlink', function (givenFileName) {
      const fileName = _path2.default.relative(resolvedSourceDirectory, givenFileName);

      const outputFile = _path2.default.join(config.outputDirectory, fileName);

      _sbFs2.default.unlink(outputFile).catch(function () {
        /* No Op */
      });
    });
    yield transformationQueue.onIdle();
    debounceExecute();
  });

  return function main(_x) {
    return _ref.apply(this, arguments);
  };
})();

var _os = require("os");

var _os2 = _interopRequireDefault(_os);

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _sbFs = require("sb-fs");

var _sbFs2 = _interopRequireDefault(_sbFs);

var _chalk = require("chalk");

var _chalk2 = _interopRequireDefault(_chalk);

var _pQueue = require("p-queue");

var _pQueue2 = _interopRequireDefault(_pQueue);

var _makeDir = require("make-dir");

var _makeDir2 = _interopRequireDefault(_makeDir);

var _chokidar = require("chokidar");

var _chokidar2 = _interopRequireDefault(_chokidar);

var _debounce = require("lodash/debounce");

var _debounce2 = _interopRequireDefault(_debounce);

var _child_process = require("child_process");

var _child_process2 = _interopRequireDefault(_child_process);

var _package = require("../package.json");

var _package2 = _interopRequireDefault(_package);

var _iterate = require("./iterate");

var _iterate2 = _interopRequireDefault(_iterate);

var _helpers = require("./helpers");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

exports.default = main;