"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

let iterate = (() => {
  var _ref = _asyncToGenerator(function* ({
    rootDirectory,
    sourceDirectory,
    outputDirectory,
    ignored,
    keepExtraFiles,
    filesToKeep,
    callback
  }) {
    const contents = yield _sbFs2.default.readdir(sourceDirectory);
    let outputStats;
    let outputDirectoryExists = false;

    try {
      outputStats = yield _sbFs2.default.stat(outputDirectory);
    } catch (_) {
      /* No Op */
    }

    if (outputStats) {
      if (outputStats.isDirectory()) {
        outputDirectoryExists = true;
      } else {
        yield (0, _del2.default)(outputDirectory);
        outputStats = null;
      }
    }

    const outputContents = outputStats ? yield _sbFs2.default.readdir(outputDirectory) : [];

    if (!keepExtraFiles) {
      const whitelist = filesToKeep(contents);
      const filesToDelete = outputContents.filter(function (item) {
        return !whitelist.includes(item);
      }).map(function (item) {
        return _path2.default.resolve(outputDirectory, item);
      });
      yield Promise.all(filesToDelete.map(function (item) {
        return (0, _del2.default)(item);
      }));
    }

    yield (0, _pMap2.default)(contents, (() => {
      var _ref2 = _asyncToGenerator(function* (fileName) {
        const filePath = _path2.default.join(sourceDirectory, fileName);

        const stat = yield _sbFs2.default.lstat(filePath);

        if (stat.isSymbolicLink()) {
          // NOTE: We ignore symlinks
          return;
        }

        if ((0, _anymatch2.default)(ignored, fileName) || (0, _anymatch2.default)(ignored, filePath) || (0, _anymatch2.default)(ignored, _path2.default.relative(rootDirectory, filePath))) {
          // NOTE: We ignore ignored files
          return;
        }

        if (stat.isFile()) {
          if (!fileName.endsWith('.js')) return;

          if (!outputDirectoryExists) {
            yield (0, _makeDir2.default)(outputDirectory);
            outputDirectoryExists = true;
          }

          yield callback(filePath, _path2.default.join(outputDirectory, fileName), stat);
        } else if (stat.isDirectory()) {
          yield iterate({
            rootDirectory,
            sourceDirectory: _path2.default.join(sourceDirectory, fileName),
            outputDirectory: _path2.default.join(outputDirectory, fileName),
            ignored,
            keepExtraFiles,
            filesToKeep,
            callback
          });
        }
      });

      return function (_x2) {
        return _ref2.apply(this, arguments);
      };
    })());
  });

  return function iterate(_x) {
    return _ref.apply(this, arguments);
  };
})();

var _del = require("del");

var _del2 = _interopRequireDefault(_del);

var _sbFs = require("sb-fs");

var _sbFs2 = _interopRequireDefault(_sbFs);

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _pMap = require("p-map");

var _pMap2 = _interopRequireDefault(_pMap);

var _makeDir = require("make-dir");

var _makeDir2 = _interopRequireDefault(_makeDir);

var _anymatch = require("anymatch");

var _anymatch2 = _interopRequireDefault(_anymatch);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

exports.default = iterate;